/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.cassisd.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.omp.irap.cassis.common.CassisDecimalFormat;
import eu.omp.irap.cassis.common.Formula;
import eu.omp.irap.cassis.common.LineDescription;
import eu.omp.irap.cassis.common.MoleculeDescription;
import eu.omp.irap.cassis.common.TypeFrequency;
import eu.omp.irap.cassis.common.axes.UNIT;
import eu.omp.irap.cassis.common.axes.XAxisCassis;
import eu.omp.irap.cassis.common.axes.XAxisCassisException;
import eu.omp.irap.cassis.common.axes.XAxisFrequency;
import eu.omp.irap.cassis.database.access.AccessDataBase;
import eu.omp.irap.cassis.database.access.DataBaseConnection;
import eu.omp.irap.cassis.common.LineDescriptionDB;
import eu.omp.irap.cassis.database.access.MoleculeDescriptionDB;
import eu.omp.irap.cassis.database.access.SimpleMoleculeDescriptionDB;
import eu.omp.irap.cassis.database.access.VamdcDataBaseConnection;

/**
 * @author glorian
 *
 */
public class OtherSpeciesModel {

	private static String lineIdentificationTemplate = "[[molNameValue]] [[quanticNumberValue]] [[tagLabel]] [[tagValue]]\n"
			+ "[[waveLabel]] [[waveValue]][[waveErrorLabelBegin]][[waveErrorValue]][[waveErrorLabelEnd]][[waveUnit]]\n"
			+ "[[elowLabel]] [[elowValue]] [[elowUnit]] [[eupLabel]] [[eupValue]] [[eupUnit]]"
			+ " [[aijLabel]] [[aijValue]]"
			+ " [[gupLabel]] [[gupValue]]\n"
			+ "[[shiftWaveLabel]] [[shiftWaveValue]] [[shiftWaveUnit]]"
			+ " [[commentLabel]] [[commentValue]]\n"
			+ "[[citationLabel]] [[citationValue]]\n"
			+ "[[otherWave]]";

	private OtherSpeciesParameters treshold ;
	private double freqMinMhz=0;
	private double freqMaxMhz=0;
	private XAxisCassis xAxisCassis= XAxisCassis.getXAxisCassis(UNIT.MHZ);
	private static final Logger LOGGER = LoggerFactory.getLogger(OtherSpeciesModel.class);


	/**
	 *
	 * @param waveMin
	 * @param waveMax
	 * @param unit
	 * @param treshold
	 * @throws XAxisCassisException
	 */
	public OtherSpeciesModel(double waveMin, double waveMax, UNIT unit,
			OtherSpeciesParameters treshold) throws XAxisCassisException {
		if (UNIT.isVelocity(unit)) {
			throw new XAxisCassisException("Error with velocity axis, You must provide "
					+ "a XAxisCassis object with the referecny Frequency");
		}

		init(waveMin, waveMax, XAxisCassis.getXAxisCassis(unit), treshold);

	}

	/**
	 *
	 * @param waveMin
	 * @param waveMax
	 * @param axis
	 * @param treshold
	 */
	public OtherSpeciesModel(double waveMin, double waveMax, XAxisCassis axis,
			OtherSpeciesParameters treshold) throws XAxisCassisException {
		init(waveMin, waveMax, axis, treshold);
	}

	/**
	 *
	 * @param frequencyMin
	 * @param frequencyMax
	 * @param otherThresEupMin
	 * @param otherThresEupMax
	 * @param otherThresAij
	 * @param otherTreshAijMax
	 * @param vlsrPlot
	 * @param vlsrData
	 * @param modeSignal
	 * @param comment
	 * @param typeFrequency
	 */
	public OtherSpeciesModel(double frequencyMin, double frequencyMax, double otherThresEupMin,
			double otherThresEupMax, double otherThresAij, double otherTreshAijMax, double vlsrPlot,
			double vlsrData,
			boolean modeSignal, String comment, TypeFrequency typeFrequency) {
		init(frequencyMin, frequencyMax, XAxisCassis.getXAxisCassis(UNIT.MHZ),
				new OtherSpeciesParameters(otherThresEupMin, otherThresEupMax, otherThresAij,
				otherTreshAijMax, vlsrPlot, vlsrData, modeSignal,
				comment, typeFrequency));

	}
	/**
	 * @param waveMin
	 * @param waveMax
	 * @param axis
	 * @param treshold
	 */
	private void init(double waveMin, double waveMax, XAxisCassis axis, OtherSpeciesParameters treshold) {
		try {
			this.xAxisCassis = axis.clone();
		} catch (CloneNotSupportedException e) {
			LOGGER.error(e.getMessage());
		}
		this.treshold = treshold;
		double range[] = new double[] {waveMin, waveMax};
		double freqRangeInMhz[] = XAxisCassis.convert(range,xAxisCassis,
				XAxisFrequency.getXAxisFrequency(UNIT.MHZ));
		this.freqMinMhz = freqRangeInMhz[0];
		this.freqMaxMhz = freqRangeInMhz[1];

		double shiftData = 0;
		if (TypeFrequency.REST.equals(treshold.getTypeFrequency())) {
			shiftData = treshold.getVlsrData();
		}

		if (treshold.getShiftWaveType().equals("Redshift")){
			this.freqMinMhz = this.freqMinMhz / (1 + shiftData - treshold.getVlsrPlot());
			this.freqMaxMhz = this.freqMaxMhz / (1 + shiftData - treshold.getVlsrPlot());

		} else   {
			this.freqMinMhz = Formula.calcFreqWithVlsr(freqMinMhz, shiftData - treshold.getVlsrPlot(), freqMinMhz);
			this.freqMaxMhz = Formula.calcFreqWithVlsr(freqMaxMhz, shiftData - treshold.getVlsrPlot(), freqMaxMhz);
		}
	}

	public List<LineDescription> createLines(List<MoleculeDescription> mols) {
		List<SimpleMoleculeDescriptionDB> listSimpleMolDB = new ArrayList<>(500);
		DataBaseConnection db = AccessDataBase.getDataBaseConnection();

		if (mols != null) {
			for (MoleculeDescription mol : mols) {
				if (mol.isCompute()) {
					listSimpleMolDB.add(db.getSimpleMolecule(mol.getSpeciesId()));
				}
			}
		}

		if (listSimpleMolDB.size() == 1) {
			return getListOfLines(listSimpleMolDB.get(0));
		} else {
			return getListOfLines(listSimpleMolDB);
		}
	}

	private List<LineDescription> getListOfLines(List<SimpleMoleculeDescriptionDB> listSimpleMolDB) {
		DataBaseConnection db = AccessDataBase.getDataBaseConnection();
		List<LineDescriptionDB> lines = db.getLineDescriptionDB(listSimpleMolDB, freqMinMhz, freqMaxMhz, 
				treshold.getEnergyLineFilter(), treshold.getAijLineStrengthFilter());
		return computeOtherSpecies(lines);
	}

	private List<LineDescription> getListOfLines(SimpleMoleculeDescriptionDB simpleMolDB) {
		DataBaseConnection db = AccessDataBase.getDataBaseConnection();
		List<LineDescriptionDB> lines = db.getLineDescriptionDB(simpleMolDB, this.freqMinMhz, this.freqMaxMhz, 
				treshold.getEnergyLineFilter(), treshold.getAijLineStrengthFilter());
		return computeOtherSpecies(lines);
	}

	private List<LineDescription> computeOtherSpecies(List<LineDescriptionDB> lines) {
		List<LineDescription> listOfLines = new ArrayList<>(lines.size());
		for (LineDescriptionDB line : lines) {
			double fMHzTrans = line.getFrequency();
			double errTrans = line.getError();
			int iguTrans = line.getIgu();
			double aijOnNu2 = Formula.calcAijOnNu2(line.getAint(), iguTrans, fMHzTrans);

			double wave = xAxisCassis.convertFromMhzFreq(fMHzTrans);
			double waveError = xAxisCassis.convertDeltaFromMhzFreq(fMHzTrans-errTrans, fMHzTrans+errTrans);

			HashMap<String, String> map = getParameterValueMapping(line,
					AccessDataBase.getDataBaseConnection().getMolName(line.getSpeciesId()), wave,waveError);

			String id = replaceLineIdentificationValue(map);

			LineDescription lineDes = new LineDescription(line);
			lineDes.setIdentification(id);
			lineDes.setAijOnNu2(aijOnNu2);
			lineDes.setVlsr(treshold.getVlsrPlot());
			lineDes.setVlsrData(treshold.getVlsrData());
			lineDes.setMolName(AccessDataBase.getDataBaseConnection().getMolName(line.getSpeciesId()));
			double shiftData = 0;
			if (TypeFrequency.REST.equals(treshold.getTypeFrequency())){
				shiftData = treshold.getVlsrData();
			}
			if (treshold.getShiftWaveType().equals("Redshift")){
				lineDes.setFreqCompute(fMHzTrans / (1 + treshold.getVlsrPlot()-shiftData));
			} else   {
				lineDes.setFreqCompute(Formula.calcFreqWithVlsr(fMHzTrans, treshold.getVlsrPlot() - shiftData, fMHzTrans));
			}

			lineDes.setDoubleSideBand(!treshold.isModeSignal());

			listOfLines.add(lineDes);
		}
		return listOfLines;
	}

	/**
	 * Create the mapping between the template otherSpecies line identification
	 * and the parameter of the line
	 * @param line
	 * @param molName
	 * @param wave
	 * @param waveError
	 * @return the mapping
	 */
	private HashMap<String, String> getParameterValueMapping(
			LineDescriptionDB line, String molName, double wave, double waveError) {
		HashMap<String, String> map = new HashMap<String, String>();
		map.put("molNameValue", molName);
		map.put("quanticNumberValue", MoleculeDescriptionDB.constitueLine(line.getQuanticNumbers()));
		map.put("tagLabel", "Tag =");
		map.put("tagValue", String.valueOf(line.getSpeciesId()));
		map.put("waveLabel", xAxisCassis.getAxis().getSymbol()+" =");
		map.put("waveValue", CassisDecimalFormat.FREQUENCY_FORMAT.format(wave));
		map.put("waveErrorLabelBegin", "(±");
		map.put("waveErrorValue", CassisDecimalFormat.FREQUENCY_FORMAT.format(waveError));
		map.put("waveUnit", xAxisCassis.getUnit().getValString());
		map.put("waveErrorLabelEnd", ") ");
		if (line.getComment()!= null && !line.getComment().isEmpty()) {
			map.put("waveComment", line.getComment());
		}
		map.put("elowLabel", "Elow =");
		double elowValue = UNIT.convertEnergyLevel(line.getElow(), 
				UNIT.CM_MINUS_1, treshold.getEnergyLineFilter().getEnergyUnit());
		String elowValueString = CassisDecimalFormat.EUP_FORMAT.format(elowValue);
		if (elowValue < 10E-1) {
			elowValueString = CassisDecimalFormat.TWO_DEC_EXP_FORMAT.format(elowValue);
		}
		map.put("elowValue", elowValueString);
		map.put("elowUnit", treshold.getEnergyLineFilter().getEnergyUnit().getValString());
		
		map.put("eupLabel", "Eup =");
		double eupValue = UNIT.convertEnergyLevel(line.getEup(), 
				UNIT.KELVIN, treshold.getEnergyLineFilter().getEnergyUnit());
		String elupValueString = CassisDecimalFormat.EUP_FORMAT.format(eupValue);
		if (eupValue < 10E-1) {
			elupValueString = CassisDecimalFormat.TWO_DEC_EXP_FORMAT.format(eupValue);
		}
		map.put("eupValue", elupValueString);
		map.put("eupUnit", treshold.getEnergyLineFilter().getEnergyUnit().getValString());
		
		map.put("aijLabel", "A =");
		map.put("aijValue", CassisDecimalFormat.AIJ_FORMAT.format(line.getAint()));
		map.put("aijUnit", "");
		map.put("gupLabel", "Gup =");
		map.put("gupValue", String.valueOf(line.getIgu()));
		map.put("gupUnit", "");

		map.put("shiftWaveLabel", treshold.getShiftWaveType() + " =");
		map.put("shiftWaveValue", CassisDecimalFormat.TWO_DEC_FORMAT.format(treshold.getVlsrPlot()));
		if (treshold.getShiftWaveType().equals("Redshift")){
			map.put("shiftWaveUnit", "");
		} else {
			map.put("shiftWaveUnit",  "km/s");
		}

		if (this.treshold.getComment() != null && !this.treshold.getComment().isEmpty()) {
			map.put("commentLabel", "   ");
			map.put("commentValue", this.treshold.getComment());
		}
		if (line.getCitation() != null && !line.getCitation().isEmpty()) {
			map.put("citationLabel", VamdcDataBaseConnection.VAMDC_REQUEST_TOKEN+ "=");
			map.put("citationValue", line.getCitation());

		}
		if (line.getOtherFreqs() != null &&  !line.getOtherFreqs().isEmpty()) {
			map.put("otherWave", "Other frequencies proposed: " + line.getOtherFreqs());
		}
		return map;
	}


	/**
	 * Replace the template of OtherSpecies line identification
	 *
	 */
	public static void setLineIdentificationTemplate(String template) {
		lineIdentificationTemplate = template;
	}

	/**
	 * Get the template of OtherSpecies line identification
	 *
	 * @return the string of OtherSpecies line identification
	 */
	public static String getLineIdentificationTemplate() {
		return lineIdentificationTemplate;
	}

	/**
	 * Format template with the values of a line.
	 *
	 * @param map The values of the lines.
	 * @return the formated string from the template for the given line values.
	 */
	private static String replaceLineIdentificationValue(HashMap<String, String> map) {
		StringBuilder sb = new StringBuilder(lineIdentificationTemplate.length());
		int crtIdx = -1;
		int idx = 0;
		while (idx < lineIdentificationTemplate.length() - 1) {
			String str = lineIdentificationTemplate.substring(idx);
			crtIdx = str.indexOf("[[");
			if (crtIdx == -1) {
				sb.append(str);
				break;
			} else {
				sb.append(str.substring(0, crtIdx));
				int next = str.indexOf("]]");
				if (next >= 0) {
					String key = str.substring(crtIdx + 2, next);
					if (map.containsKey(key)) {
						sb.append(map.get(key));
					}
					idx += next + 2;
				} else {
					idx += crtIdx;
				}
			}
		}
		return sb.toString().trim();
	}

	/**
	 * @see java.lang.Object#toString()
	 */
	@Override
	public String toString() {
		return "OtherSpeciesModel [frequencyMin=" + this.freqMinMhz + ", frequencyMax="
				+ this.freqMaxMhz + ", otherThresEupMin=" + treshold.getOtherThresEupMin() + ", otherThresEupMax=" + treshold.getOtherThresEupMax()
				+ ", otherThresAijMin=" + treshold.getOtherThresAijMin() + ", otherTreshAijMax=" + treshold.getOtherTreshAijMax() + ", vlsr="
				+ treshold.getVlsrPlot() + ", modeSignal=" + treshold.isModeSignal() + ", comment=" + treshold.getComment() + "]";
	}
}
