/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.configuration;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.List;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTextArea;
import javax.swing.WindowConstants;
import javax.swing.border.TitledBorder;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import eu.omp.irap.cassis.common.LineDescription;
import eu.omp.irap.cassis.common.LineDescriptionDB;
import eu.omp.irap.cassis.common.axes.UNIT;
import eu.omp.irap.cassis.common.axes.XAxisCassis;
import eu.omp.irap.cassis.gui.otherspecies.SpeciesLabelConstants;

/**
 * Lines template configuration class.
 *
 * @author M. Boiziot
 */
public class LinesTemplateDialog extends JDialog {

	private static final long serialVersionUID = 1L;

	private String textBase; // This is the setting value as defined before.
	private String textDefault; // This is the default value for the setting.
	private JButton resetChangesButton;
	private JButton setDefautValueButton;
	private JButton validateButton;
	private JTextArea inputTextArea;
	private JTextArea outputTextArea;
	private JPanel mainPanel;
	private JPanel buttonsPanel;
	private List<JButton> constantButtons;


	/**
	 * Constructor.
	 *
	 * @param owner The owner of the dialog.
	 * @param textBase The current setting value.
	 * @param textDefault The default CASSIS setting value.
	 */
	public LinesTemplateDialog(Window owner, String textBase, String textDefault) {
		super(owner, ModalityType.APPLICATION_MODAL);
		this.textBase = textBase;
		this.textDefault = textDefault;
		setTitle("Lines template");
		setContentPane(getMainPanel());
		setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
		setSize(new Dimension(1000, 560));
	}

	/**
	 * Create if needed then return the reset changes button.
	 *
	 * @return the reset changes button.
	 */
	public JButton getResetChangesButton() {
		if (resetChangesButton == null) {
			resetChangesButton = new JButton("Reset changes");
			resetChangesButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					getInputTextArea().setText(textBase);
					refresh();
				}
			});
		}
		return resetChangesButton;
	}

	/**
	 * Create if needed then return the default values button.
	 *
	 * @return the default values button.
	 */
	public JButton getSetDefaultValuesButton() {
		if (setDefautValueButton == null) {
			setDefautValueButton = new JButton("Set default values");
			setDefautValueButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					getInputTextArea().setText(textDefault);
					refresh();
				}
			});
		}
		return setDefautValueButton;
	}

	/**
	 * Create if needed then return the validate button.
	 *
	 * @return the validate button.
	 */
	public JButton getValidateButton() {
		if (validateButton == null) {
			validateButton = new JButton("Validate");
			validateButton.addActionListener(new ActionListener() {
				@Override
				public void actionPerformed(ActionEvent e) {
					LinesTemplateDialog.this.dispose();
				}
			});
		}
		return validateButton;
	}

	/**
	 * Create then return the main panel.
	 *
	 * @return the main panel.
	 */
	public JPanel getMainPanel() {
		if (mainPanel == null) {
			mainPanel = new JPanel(new BorderLayout());

			JPanel centerPanel = new JPanel(new GridLayout(0,1));
			centerPanel.add(getInputTextArea());
			centerPanel.add(getOutputTextArea());

			JPanel bottomPanel = new JPanel();
			bottomPanel.add(getResetChangesButton());
			bottomPanel.add(getSetDefaultValuesButton());
			bottomPanel.add(getValidateButton());

			mainPanel.add(centerPanel, BorderLayout.CENTER);
			mainPanel.add(getButtonsPanel(), BorderLayout.EAST);
			mainPanel.add(bottomPanel, BorderLayout.SOUTH);
		}
		return mainPanel;
	}

	/**
	 * Create then return the right buttons panel.
	 *
	 * @return the right buttons panel.
	 */
	private JPanel getButtonsPanel() {
		if (buttonsPanel == null) {
			buttonsPanel = new JPanel(new GridLayout(0,1));
			for (JButton button : getConstantsButtons()) {
				buttonsPanel.add(button);
			}
		}
		return buttonsPanel;
	}

	/**
	 * Create if needed then return the constants buttons.
	 *
	 * @return the constants buttons.
	 */
	private List<JButton> getConstantsButtons() {
		if (constantButtons == null) {
			LineConstantParameter[] constantParameters = getAllLineConstantParameters();
			constantButtons = new ArrayList<>(constantParameters.length);
			for (LineConstantParameter lineConstantParameter : constantParameters) {
				constantButtons.add(createButton(lineConstantParameter));
			}
		}
		return constantButtons;
	}

	/**
	 * Create then return the button for the given LineConstantParameter.
	 *
	 * @param lineConstantParameter the LineConstantParameter.
	 * @return the button.
	 */
	private JButton createButton(LineConstantParameter lineConstantParameter) {
		JButton button = new JButton(lineConstantParameter.textButton);
		button.setActionCommand("[[" + lineConstantParameter.constant + "]]");
		button.setToolTipText(lineConstantParameter.tooltip);
		button.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				String tag = button.getActionCommand();
				getInputTextArea().append(tag);
			}
		});
		return button;
	}

	/**
	 * Create if needed then return the input text area.
	 *
	 * @return the input text area.
	 */
	private JTextArea getInputTextArea() {
		if (inputTextArea == null) {
			inputTextArea = new JTextArea(this.textBase);
			TitledBorder border = BorderFactory.createTitledBorder("Template");
			border.setTitleJustification(TitledBorder.CENTER);
			inputTextArea.setBorder(border);
			inputTextArea.getDocument().addDocumentListener(new DocumentListener() {
				@Override
				public void removeUpdate(DocumentEvent e) {
					refresh();
				}
				@Override
				public void insertUpdate(DocumentEvent e) {
					refresh();
				}
				@Override
				public void changedUpdate(DocumentEvent e) {
					refresh();
				}
			});
		}
		return inputTextArea;
	}

	/**
	 * Create if needed then return the output text area.
	 *
	 * @return the output text area.
	 */
	private JTextArea getOutputTextArea() {
		if (outputTextArea == null) {
			outputTextArea = new JTextArea();
			outputTextArea.setEditable(false);
			outputTextArea.setBackground(Color.LIGHT_GRAY);
			TitledBorder border = BorderFactory.createTitledBorder("Example result");
			border.setTitleJustification(TitledBorder.CENTER);
			outputTextArea.setBorder(border);
			refresh();
		}
		return outputTextArea;
	}

	/**
	 * Refresh the output text area from the template in input text area.
	 */
	private void refresh() {
		String rawLine = getTemplate();
		LineDescription sampleLine = getSampleLine();
		SpeciesLabelConstants.updateLine(sampleLine, sampleLine.getObsFrequency(),
				5E-7, UNIT.KELVIN, 3.8, XAxisCassis.getXAxisFrequency(), rawLine);
		getOutputTextArea().setText(sampleLine.getIdentification());
	}

	/**
	 * Create the return a dummy CO line.
	 *
	 * @return a dummy CO line.
	 */
	private LineDescription getSampleLine() {

		LineDescriptionDB lineDb = new LineDescriptionDB("28503", 115271.2021, "1 _ 0",
				1.0E-4, 7.20360334988053E-8, 3, 5.532147013851108);
		LineDescription line = new LineDescription(lineDb,0.0,
				0.0, null, null,1.6264076248661992E-29);
		line.setMolName("CO, v=0");

		List<String> otherFreqs = new ArrayList<>(1);
		otherFreqs.add("115271.2018±5.0E-4");
		lineDb.setOtherFreqs(otherFreqs);
		return line;
	}

	/**
	 * Return the template.
	 *
	 * @return the template.
	 */
	public String getTemplate() {
		return getInputTextArea().getText();
	}

	/**
	 * Create then return all LineConstantParameters.
	 *
	 * @return all LineConstantParameters.
	 */
	private LineConstantParameter[] getAllLineConstantParameters() {
		return new LineConstantParameter[] {
				new LineConstantParameter(SpeciesLabelConstants.AIJ_UNIT, "Aij unit", "This is replaced by the aij unit"),
				new LineConstantParameter(SpeciesLabelConstants.AIJ_VALUE, "Aij value", "This is replaced by the aij value"),
				new LineConstantParameter(SpeciesLabelConstants.CITATION_LABEL, "Citation label", "This is replaced by the citation label, only if available"),
				new LineConstantParameter(SpeciesLabelConstants.CITATION_VALUE, "Citation value", "This is replaced by the citation, only if available"),
				new LineConstantParameter(SpeciesLabelConstants.ELOW_UNIT, "Elow unit", "This is replaced by the elow unit"),
				new LineConstantParameter(SpeciesLabelConstants.ELOW_VALUE, "Elow value", "This is replaced by the elow value"),
				new LineConstantParameter(SpeciesLabelConstants.EUP_UNIT, "Eup unit", "This is replaced by the eup unit"),
				new LineConstantParameter(SpeciesLabelConstants.EUP_VALUE, "Eup value", "This is replaced by the eup value"),
				new LineConstantParameter(SpeciesLabelConstants.GUP_UNIT, "Gup unit", "This is replaced by the gup unit"),
				new LineConstantParameter(SpeciesLabelConstants.GUP_VALUE, "Gup value", "This is replaced by the gup value"),
				new LineConstantParameter(SpeciesLabelConstants.MOL_NAME_VALUE, "Species name", "This is replaced by the species name"),
				new LineConstantParameter(SpeciesLabelConstants.OTHER_WAVE, "Other waves avalaible", "This is replaced by the other waves avalaible"),
				new LineConstantParameter(SpeciesLabelConstants.QUANTIC_NUMBER_VALUE, "Quantum number value", "This is replaced by the quantum number"),
				new LineConstantParameter(SpeciesLabelConstants.SHIFT_WAVE_UNIT, "Wave shift unit", "This is replaced by the shift wave unit"),
				new LineConstantParameter(SpeciesLabelConstants.SHIFT_WAVE_VALUE, "Wave shift value", "This is replaced by the shift wave value"),
				new LineConstantParameter(SpeciesLabelConstants.TAG_VALUE, "Tag value", "This is replaced by the tag value"),
				new LineConstantParameter(SpeciesLabelConstants.WAVE_ERROR_LABEL_BEGIN, "Wave error label begin", "This is replaced by the begin of the wave error label"),
				new LineConstantParameter(SpeciesLabelConstants.WAVE_ERROR_LABEL_END, "Wave error label end", "This is replaced by the end of the wave error label"),
				new LineConstantParameter(SpeciesLabelConstants.WAVE_ERROR_VALUE, "Wave error value", "This is replaced by the wave error value"),
				new LineConstantParameter(SpeciesLabelConstants.WAVE_LABEL, "Wave label", "This is replaced by the wave label"),
				new LineConstantParameter(SpeciesLabelConstants.WAVE_UNIT, "Wave unit", "This is replaced by the wave unit"),
				new LineConstantParameter(SpeciesLabelConstants.WAVE_VALUE, "Wave value", "This is replaced by the wave value")
		};
	}



	/**
	 * Define the elements needed for a line constant button.
	 *
	 * @author M. Boiziot
	 */
	private class LineConstantParameter {
		private String constant;
		private String textButton;
		private String tooltip;


		/**
		 * Constructor.
		 *
		 * @param constant The constant.
		 * @param textButton The text to set for the button.
		 * @param tooltip The tool tip value to set for the button.
		 */
		private LineConstantParameter(String constant, String textButton, String tooltip) {
			this.constant = constant;
			this.textButton = textButton;
			this.tooltip = tooltip;
		}
	}
}
