/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.configuration;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.BorderFactory;
import javax.swing.JComboBox;
import javax.swing.JPanel;

import eu.omp.irap.cassis.gui.PanelView;
import eu.omp.irap.cassis.gui.menu.action.EpnTapAction;
import eu.omp.irap.cassis.properties.Software;
import eu.omp.irap.cassis.properties.UserConfiguration;

/**
 * VO tab for the configuration.
 *
 * @author M. Boiziot
 */
public class VoConfigurationView extends JPanel {

	private static final long serialVersionUID = -7238609379427077906L;

	private JComboBox<Boolean> autoStartSampCombobox;
	private JComboBox<Boolean> useHardcodedRegistryComboBox;
	private JComboBox<Boolean> displaySystrayIconSampComboBox;


	/**
	 * Constructor.
	 */
	public VoConfigurationView() {
		super(new GridBagLayout());
		setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));
		createContent();
	}

	/**
	 * Create the content of the panel.
	 */
	private void createContent() {
		GridBagConstraints gbc = new GridBagConstraints();
		UserConfigurationUtils.createLine(this, gbc, "Autostart Samp:",
				null, getSampComboBox(), null, 0);
		UserConfigurationUtils.createLine(this, gbc, "Display system tray icon for SAMP:",
				null, getDisplaySystrayIconSampComboBox(), null, 1);
		UserConfigurationUtils.createLine(this, gbc,
				"Use hardcoded EPN-TAP registry:",
				"If set to true, EPN-TAP will use an hardcoded list of services.\n"
				+ "If set to false, will use a VO registry to get the list of service",
				getUseHardcodedRegistryComboBox(), null, 2);
	}

	/**
	 * Create if needed then return JComboBox about using a hard-coded registry
	 *  for EPN-TAP.
	 *
	 * @return the JComboBox about using a hard-coded registry for EPN-TAP.
	 */
	private JComboBox<Boolean> getUseHardcodedRegistryComboBox() {
		if (useHardcodedRegistryComboBox == null) {
			Boolean[] booleans = new Boolean[] {true, false};
			useHardcodedRegistryComboBox = new JComboBox<>(booleans);
			useHardcodedRegistryComboBox.setSelectedItem(
					Software.getUserConfiguration().isHardcodedEpnTapRegistryUsed());
			useHardcodedRegistryComboBox.setPreferredSize(new Dimension(300, 24));
		}
		return useHardcodedRegistryComboBox;
	}

	/**
	 * Create if needed then return the Samp autostart {@link JComboBox}.
	 *
	 * @return the Samp autostart {@link JComboBox}.
	 */
	private JComboBox<Boolean> getSampComboBox() {
		if (autoStartSampCombobox == null) {
			Boolean[] booleans = new Boolean[] {true, false};
			autoStartSampCombobox = new JComboBox<>(booleans);
			autoStartSampCombobox.setSelectedItem(
					Software.getUserConfiguration().isSampOnAutoStart());
		}
		return autoStartSampCombobox;
	}

	/**
	 * Save the configuration.
	 */
	public void saveConfiguration() {
		UserConfiguration conf = Software.getUserConfiguration();
		boolean selectedValue = (Boolean) getUseHardcodedRegistryComboBox().getSelectedItem();
		boolean oldValue = conf.isHardcodedEpnTapRegistryUsed();
		boolean change = selectedValue != oldValue;
		if (change) {
			conf.setHarcodedEpnTapRegistry(selectedValue);
			EpnTapAction eta = (EpnTapAction) PanelView.getInstance()
					.getCassisActionMenu().getEpnTapAction();
			eta.dispose_and_reopen_if_needed();
		}
		conf.setSampOnAutoStart(((Boolean)
				getSampComboBox().getSelectedItem()).booleanValue());
		conf.setDisplaySystrayIconSamp(((Boolean)
				getDisplaySystrayIconSampComboBox().getSelectedItem()).booleanValue());
	}

	/**
	 * Create if needed then return the {@link JComboBox} display an icon in
	 *  the system tray for SAMP.
	 *
	 * @return the display icon in system tray for SAMP ComboBox {@link JComboBox}.
	 */
	private JComboBox<Boolean> getDisplaySystrayIconSampComboBox() {
		if (displaySystrayIconSampComboBox == null) {
			Boolean[] booleans = new Boolean[] {true, false};
			displaySystrayIconSampComboBox = new JComboBox<>(booleans);
			displaySystrayIconSampComboBox.setSelectedItem(
					Software.getUserConfiguration().isDisplaySystrayIconSamp());
		}
		return displaySystrayIconSampComboBox;
	}

}
