/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.gui.otherspecies;

import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ExecutionException;

import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;
import javax.swing.SwingWorker;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import eu.omp.irap.cassis.common.LineDescription;
import eu.omp.irap.cassis.gui.plot.line.CassisPlot;
import eu.omp.irap.cassis.properties.Software;

/**
 * Other species worker for a mosaic view (Line Spectrum/Multi Spectrum).
 */
public class SpeciesMosaicWorker extends SwingWorker<Boolean, Void>{

	private static final Logger LOGGER = LoggerFactory.getLogger(SpeciesMosaicWorker.class);
	private SpeciesMosaicInterface mosaicInterface;
	private List<CassisPlot> cassisPlots;
	private SpeciesControl control;
	private boolean haveImage;


	/**
	 * Constructor.
	 *
	 * @param mosaicInterface The SpeciesMosaicInterface.
	 * @param cassisPlots The list of CassisPlot for which the other species
	 *  computation need to be made.
	 * @param control The SpeciesControl.
	 * @param haveImage true if image is selected, false otherwise.
	 */
	public SpeciesMosaicWorker(SpeciesMosaicInterface mosaicInterface,
			List<CassisPlot> cassisPlots, SpeciesControl control, boolean haveImage)  {
		super();
		this.mosaicInterface = mosaicInterface;
		this.cassisPlots = cassisPlots;
		this.control = control;
		this.haveImage = haveImage;
	}

	/**
	 * Do the other species computation in a background thread and affect other
	 *  species to CassisPlot (in EDT).
	 */
	@Override
	protected Boolean doInBackground() throws Exception {
		LOGGER.info("Begin computing other species");
		boolean find = false;
		for (CassisPlot cassisPlot : this.cassisPlots) {
			if (isCancelled()) { return false; }
			final List<LineDescription> signalLines = computeSignalLines(cassisPlot);
			if (isCancelled()) { return false; }
			final List<LineDescription> imageLines = computeImageLines(cassisPlot);
			if (!find && (!signalLines.isEmpty() || !imageLines.isEmpty())) {
				find = true;
			}
			if (isCancelled()) { return false; }
			handleResultInEdt(cassisPlot, signalLines, imageLines);
		}
		LOGGER.info("End computing other species");
		return find;
	}

	/**
	 * Handle the other species computation for a given CassisPlot by affecting
	 *  the result to the CassisPlot in EDT.
	 *
	 * @param cassisPlot The CassisPlot.
	 * @param signalLines The signal lines.
	 * @param imageLines The image lines.
	 */
	private void handleResultInEdt(final CassisPlot cassisPlot,
			final List<LineDescription> signalLines,
			final List<LineDescription> imageLines) {
		try {
			SwingUtilities.invokeAndWait(new Runnable() {
				@Override
				public void run() {
					cassisPlot.setOtherSpeciesSignal(signalLines);
					if (!imageLines.isEmpty()) {
						cassisPlot.setOtherSpeciesImage(imageLines);
					}
				}
			});
		} catch (InvocationTargetException | InterruptedException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Compute then return the signal lines for a given CassisPlot.
	 *
	 * @param cassisPlot The CassisPlot.
	 * @return The signal lines (never null, eventually an empty List).
	 */
	private List<LineDescription> computeSignalLines(CassisPlot cassisPlot) {
		List<LineDescription> signalLines =  control.computeOtherSpecies(
				cassisPlot.getDataSpectrum().getFrequencySignalMin(),
				cassisPlot.getDataSpectrum().getFrequencySignalMax(),
				true,
				"(signal)");
		if (signalLines == null) {
			signalLines = new ArrayList<LineDescription>(0);
		}
		return signalLines;
	}

	/**
	 * Compute then return the images lines for a given CassisPlot if needed
	 *  and possible (IE: the haveImage is true and the spectrum have a non
	 *  NaN LO frequency).
	 *
	 * @param cassisPlot The CassisPlot.
	 * @return The image lines (never null, eventually an empty List).
	 */
	private List<LineDescription> computeImageLines(CassisPlot cassisPlot) {
		List<LineDescription> imageLines = null;
		if (haveImage && !Double.isNaN(cassisPlot.getDataSpectrum().getLoFreq())) {
			imageLines = control.computeOtherSpecies(
					cassisPlot.getDataSpectrum().getFrequencySignalMax() +
					2 * (cassisPlot.getDataSpectrum().getLoFreq()-
							cassisPlot.getDataSpectrum().getFrequencySignalMax()),
					cassisPlot.getDataSpectrum().getFrequencySignalMin() +
					2 * (cassisPlot.getDataSpectrum().getLoFreq() -
						 cassisPlot.getDataSpectrum().getFrequencySignalMin()),
					false,
					"(image)");
		}
		if (imageLines == null) {
			imageLines = new ArrayList<LineDescription>(0);
		}
		return imageLines;
	}

	/**
	 * Handle the result in the EDT:
	 * 	- Delete the other species if the worker has been stopped
	 *  - Display the line if there is
	 *  - Display an error message if there is no lines.
	 */
	@Override
	protected void done() {
		if (isCancelled()) {
			cleanUpOtherSpecies();
		} else {
			try {
				boolean find = get();
				if (find) {
					mosaicInterface.displayOtherSpecies();
				} else if (!Software.getUserConfiguration().isAutoOtherSpeciesDisplayed() ||
						   !control.getModel().getCassisTableMoleculeModel().getSelectedMolecules().isEmpty()){
					JOptionPane.showMessageDialog(mosaicInterface.getView(),
							"No transition found. Check your thresholds and/or template if necessary.",
							"Other Species", JOptionPane.WARNING_MESSAGE);
					cleanUpOtherSpecies();
				}
			} catch (InterruptedException | ExecutionException e) {
				// Nothing to do here, ignore it.
			}
		}
		control.getView().setDone();
	}

	/**
	 * Clean up the other species : remove then and display the dataset.
	 */
	private void cleanUpOtherSpecies() {
		mosaicInterface.removeOtherSpecies();
		mosaicInterface.displayDataset();
	}

	/**
	 * Stop the worker. Note: while it handle some case to clean up the operation,
	 *  it will not stop currently running database query or I/O.
	 */
	public void stop() {
		try {
			this.cancel(true);
		} catch (Exception e) {
			// Nothing to do here, ignore it.
		}
		try {
			Thread.currentThread().interrupt();
		} catch (Exception e) {
			// Nothing to do here, ignore it.
		}
	}

}
