/**
 * Copyright IRAP (CNRS/UPS/CNES)
 * 
 * The complete list of authors can be seen on CASSIS website at
 * the following URL "http://cassis.irap.omp.eu/?page=authors".
 * 
 * This software is a computer program whose purpose is to display, analyze
 * and create model of astrophysical spectra. It allow to use database and
 * related virtual observatory tools.
 * 
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use, 
 * modify and/ or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info". 
 * 
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability. 
 * 
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or 
 * data to be ensured and,  more generally, to use and operate it in the 
 * same conditions as regards security. 
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */

package eu.omp.irap.cassis.properties;

/**
 * Class to handle CASSIS versionning from a String.
 *
 * @author M. Boiziot
 */
public class CassisVersionManager {

	private static CassisVersionManager instance;

	private boolean beta;
	private boolean snapshot;
	private boolean release;
	private String simpleVersion;
	private String rawFullVersion;
	private String date;
	private String buildNumber;


	/**
	 * The constructor. Should not be called outside getInstance().
	 * Staying public to make testing easier.
	 *
	 * @param rawFullVersion The String of the version. Something like:
	 * "Cassis6.0-210413-build6882".
	 */
	public CassisVersionManager(String rawFullVersion) {
		decodeVersion(rawFullVersion);
	}

	/**
	 * Create if needed then return the instance of CassisVersionManager.
	 *
	 * @return the instance of CassisVersionManager.
	 */
	public static CassisVersionManager getInstance() {
		if (instance == null) {
			instance = new CassisVersionManager(CassisVersion.VERSION);
		}
		return instance;
	}

	/**
	 * Decode the version from the given String.
	 *
	 * @param rawFullVersion The String of the version. Something like:
	 * "Cassis6.0-210413-build6882".
	 */
	private void decodeVersion(String rawFullVersion) {
		this.rawFullVersion = rawFullVersion;
		String rawVersionLowerCase = rawFullVersion.toLowerCase();

		// Check type of build.
		this.beta = rawVersionLowerCase.contains("beta");
		this.snapshot = rawVersionLowerCase.contains("snapshot");
		this.release = !snapshot && !beta;

		// Remove unneeded string
		String cleanedVersion = rawVersionLowerCase
				.replaceAll("cassis-", "")
				.replaceAll("cassis", "")
				.replaceAll("snapshot", "")
				.replaceAll("beta", "")
				.replaceAll("build", "");

		String[] ss = cleanedVersion.split("-");
		String version = "???";
		String date = "???";
		String build = "???";

		if (ss.length == 3) {
			version = ss[0].trim();
			date = ss[1].trim();
			build = ss[2].trim();
		}
		this.simpleVersion = version;
		this.buildNumber = build;
		this.date = date;
	}

	/**
	 * Return if the version is a beta.
	 *
	 * @return true if the version is a beta, false otherwise.
	 */
	public boolean isBeta() {
		return beta;
	}

	/**
	 * Return if the version is a snapshot.
	 *
	 * @return true if the version is a snapshot, false otherwise.
	 */
	public boolean isSnapshot() {
		return snapshot;
	}

	/**
	 * Return if the version is a release.
	 *
	 * @return true if the version is a release, false otherwise.
	 */
	public boolean isRelease() {
		return release;
	}

	/**
	 * Return the simple version of the application (IE: 6.0, 5.1, etc.).
	 *
	 * @return the simple version of the application.
	 */
	public String getSimpleVersion() {
		return simpleVersion;
	}

	/**
	 * Return the raw version String. Something like "Cassis6.0-210413-build6882".
	 *
	 * @return the raw version String.
	 */
	public String getRawVersion() {
		return rawFullVersion;
	}

	/**
	 * Return the date of the build in "YYMMDD" format. Something like "210420".
	 *
	 * @return the date of the build.
	 */
	public String getDate() {
		return date;
	}

	/**
	 * Return the build number.
	 *
	 * @return the build number.
	 */
	public String getBuildNumber() {
		return buildNumber;
	}

	/**
	 * Return a friendly version of CASSIS without the type Online/Plugin.
	 *
	 * @return a friendly version of CASSIS without the type Online/Plugin.
	 */
	public String getFriendlyVersion() {
		StringBuilder sb = new StringBuilder();
		sb.append("CASSIS ");
		sb.append(simpleVersion);
		if (snapshot) {
			sb.append("-SNAPSHOT");
			sb.append(date);
			sb.append("-");
			sb.append("build");
			sb.append(buildNumber);
		} else if (beta) {
			sb.append(" beta");
		}
		return sb.toString();
	}

	/**
	 * Create then return an http user-agent friendly version.
	 * This is in the form of CASSIS/X.X-snapshot or CASSIS/X.Y.
	 *
	 * @return an HTTP user-agent friendly version.
	 */
	public String getHttpUserAgentVersion() {
		StringBuilder sb = new StringBuilder();
		sb.append("CASSIS/");
		sb.append(getSimpleVersion());
		if (isBeta()) {
			sb.append("-beta");
		} else if (isSnapshot()) {
			sb.append("-snapshot");
		}
		return sb.toString();
	}
}
